PeanoInt

Struct PeanoInt 

#[non_exhaustive]
#[repr(transparent)]
pub struct PeanoInt(pub u64);
A peano integer wrapping a 64-bits integer.

See the module explanation.

Tuple Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
0: u64

Implementations

Source

impl PeanoInt

pub fn new() -> Self

Create a new peano integer with value 0.

ghost

ensures

result.0 == 0u64

pub fn incr(self) -> Self

Increase the integer by one.

This method guarantees that increments cannot get optimized together, e.g. that

let mut x = PeanoInt::new();
for _ in 0..1_000_000 {
    x.incr();
}

Does not get optimized down to a single addition.

Since the backing integer is 64 bits long, no program could ever actually reach the point where the integer overflows.

terminates

ensures

result.0@ == self.0@ + 1

pub fn to_u64(self) -> u64

Get the underlying integer.

ghost

ensures

result == self.0

pub fn to_i64(self) -> i64

Get the underlying integer.

ghost

ensures

result@ == self.0@

pub fn to_u128(self) -> u128

Get the underlying integer.

ghost

ensures

result@ == self.0@

pub fn to_i128(self) -> i128

Get the underlying integer.

ghost

ensures

result@ == self.0@